## PSAM卡与APDU指令

### 文档说明
- APDU指令

    固定四个字节命令头（CLA INS P1 P2）和长度可变的命令体(Lc 数据域 Le)

    CLA字节指出命令的类型

    INS字节表示命令编码，P1和P2为具体命令参数

    Lc字节表示数据的长度，只有一个字节表示，取值范围为1-110.如果Lc为0表示没有数据域。

    Le表示期望卡返回的数据长度，由单字节表示，取值范围1-254字节


    一、 Write Key增加和修改密钥

    1）命令报文

    明文安装或修改密钥报文如下：

    | 代码 |值|
    |---|---|
    | CLA |80|
    |INS|D4|
    |P1| 00 -安装密钥<br/> 01-修改密钥|
    |P2|00|
    |Lc|密钥信息长度|
    |DATA|密钥信息|

    密文安装或修改密钥命令报文如下：

    | 代码 |值|
    |---|---|
    | CLA |80|
    |INS|D4|
    |P1| 00 -用于安装密钥<br/> XX-密钥类型(用于修改密钥)|
    |P2|00--用于安装密钥<br/> XX-密钥标识、版本(用于修改密钥)|
    |Lc|密钥信息长度|
    |DATA|加密后的密钥信息+4字节MAC码|

    [注1] 当密钥类型和标识都为00时，如果密钥文件为空,则表示安装卡片（或者应用）主控密钥，
    如果安装卡片（或应用）主控密钥已经存在，则表示修改卡片（或者应用的）主控密钥。

     2）命令报文数据域

    - 明文形式的数据域信息

    |密钥信息（7个字节密钥头+密钥值）|
    |--------------------------- |

    <table>
        <tr>
            <th colspan=8 bgcolor=#7B68EE>密钥头</th>
            <th>密钥值</th>
        </tr>
        <tr>
            <td>版本标识符(1)</td>
            <td>算法标识(1)</td>
            <td>密钥用途(1)</td>
            <td>使用权限(1)</td>
            <td>后续状态(1)</td>
            <td>修改权限(1)</td>
            <td>初始错误计数(1/2)</td>
            <td>当前错误计数(1/2)</td>
            <td>（02-10）</td>
        </tr>
    </table>

    [注：]

        密钥标识符（KID）：不能等于00h和FFh,同类型密钥标识符必须唯一

        版本号：同类密钥的版本，默认为等同于密钥标识

        算法标识：算法标识缺省为00,算法为3DES，如果是DES,则算法标识为01，算法标识对PIN没有意义。
        如果用户自定义了自己的算法，此处必须指明算法为08.

        使用权限：指使用某一密钥时所需满足的安全条件

        修改权限：指用Write KEY之类修改某一密钥时所需要满足的安全条件

        后续状态：只对PIN、和外部认证KEY有效。

        错误计数器：错误计数器的高半字节为初始错误计数。指明密钥可以连续错误的最多次数

        密钥用途：密钥用途为1字节，低5位为密钥类型，高3位为分散级数。

        PSAM卡支持的密钥类型：

        00 主控密钥（外部认证）

        01 卡片（应用）维护密钥，用于产生应用锁定、应用解锁、卡片锁定、和读、更新二进制、记录命令的MAC

        02 消费密钥

        03 （用户卡）PIN解锁密钥

        04 （用户卡）重装PIN密钥

        05 用户卡应用维护密钥

        06 MAC密钥

        07 加密密钥

        08 MAC、加密密钥   09 内部认证密钥  0A 超级PIN，用于解锁个人PIN

        0B 个人密钥（PIN） 用户个人密钥校验

        0C 解密密钥     0D PSAM母卡导出密钥  0E PSAM母卡导出密钥的保护密钥

        0F PIN解锁密钥  10 生成过程密钥的密钥 11 DSTK密钥

        13 PIN解锁密钥  14 RELOAD PIN 密钥  其他 系统保留








    - 安装外部认证密钥过程

        16字节的密钥值为：11223344556677881122334455667788

        7字节的密钥头：01 00 00 0F 01 3F 55(算法标识-密钥用途-使用权限-后续状态-修改权限-初始错误计数-当前错误计数)

        (1) 明文安装此密钥命令为：

        80 D4 00 00 17 01 00 00 0F 01 3F 55  11223344556677881122334455667788

        (2) 密文+MAC安装此密钥为：

        84 D4 00 00 1C + 密文 + MAC

        A、计算密文

        B、计算MAC


    - 明文+MAC形式的数据域信息

        | 明文密钥信息 |4字节MAC|
        |---|---|

        MAC是主控密钥对下数据进行MAC计算（按所列顺序）产生的：

        CLA + INS + P1 + P2+ Lc + 密钥版本号   + 算法标识   + 密钥用途  + 使用权限   + 后续状态 + 修改权限 + 错误计数器 + 密钥值


    - 密文形式的数据域信息

         | 加密后的密钥信息 |4字节MAC|
         |---|---|

         [注2] 密文安装密钥说明：

         在使用密文形式进行安装密钥时，P1,P2必须为0,命令报文数据域包括要装载的密钥密文信息和MAC

         标准密文形式的数据域信息（1C或14）

         :对于维护密钥、消费密钥、PIN解锁密钥、重装PIN密钥、用户卡应用维护密钥、MAC密钥、加密密钥、

         MAC加密密钥、解密密钥、内部认证密钥、母卡导出密钥、母卡导出保护密钥、

         (1)密钥密文信息使用主控密钥对以下数据加密（按顺序）产生的：

         1密钥用途+2密钥版本号+3算法标识（半字节使用条件[0-x] + 半字节算法标识）+4密钥值

         (2)在安装外部认证密钥过程中，密钥密文信息使用主控密钥对以下数据加密产生的：
         1密钥用途+2版本号（半个字节）+ 错误次数（半个字节）+3后续状态（半字节使用条件[0-x] + 半字节后续状态 ）+4密钥值

         (3)在安装PIN和超级PIN过程中，密文信息使用主控密钥对以下数据加密（按所列顺序）产生的：
         1密钥用途+2版本号（半个字节）+ 错误次数（半个字节）+3后续状态（半字节使用条件[0-x] + 半字节后续状态 ）+4密钥值

         PIN的长度为2~6的字节。其余补FF，直到8个字节长度；超级PIN长度为2~8个字节，其余补FF，直到8个字节长度。

         (4)扩展密文形式的数据域信息（Lc=1C或14）

     [*注*]SMARTCOS-PSAM规定：密钥标识为01的外部认证密钥为主控密钥,应用下的其他密钥钧由应用主控密钥加密安装




- APDU响应

    响应应用数据单元包括两部分：可能存在的响应数据体（应答体）和两个状态字节（应答尾部）

- PSAM说明
- 通讯协议

    ISO7816-3中定义的异步半双工字符传输协议（T=0）



### 项目结构


### 计划功能


### 如何扩展


### 注意事项